import cv2
import mediapipe as mp


class FaceDetector:
    def __init__(self):
        self.mp_face_detection = mp.solutions.face_detection
        self.face_detection = self.mp_face_detection.FaceDetection(model_selection=0, min_detection_confidence=0.5)
        self.mp_draw = mp.solutions.drawing_utils

    def find_faces(self, img, draw=True):
        """
        When the image file is read with the OpenCV function read() ,
        the order of colors is BGR (blue, green, red). So here we are converting
        BGR image to RGB
        """
        img_rgb = cv2.cvtColor(img, cv2.COLOR_BGR2RGB)
        """
        Processes an RGB image and returns a list of the detected face location data.
        It returns a NamedTuple object with a "detections" field that contains a list of the
        detected face location data.
        """
        self.results = self.face_detection.process(img_rgb)
        bboxs = []

        if self.results.detections:
            for detection in self.results.detections:
                bbox_rel = detection.location_data.relative_bounding_box
                ih, iw, ic = img.shape
                """Fitting bbox coordinates with the image shape"""
                bbox = int(bbox_rel.xmin * iw), int(bbox_rel.ymin * ih), int(bbox_rel.width * iw), int(bbox_rel.height * ih)

                bbox_info = {"bbox": bbox, "score": detection.score}

                bboxs.append(bbox_info)

                if draw:
                    img = cv2.rectangle(img, bbox, (0, 255, 0), 2)

                    cv2.putText(img, f'{int(detection.score[0] * 100)}%',
                                (bbox[0], bbox[1] - 20), cv2.FONT_HERSHEY_PLAIN,
                                2, (255, 0, 255), 2)

        return img, bboxs


def main():
    cap = cv2.VideoCapture(0)
    detector = FaceDetector()
    while True:
        success, img = cap.read()
        img, bboxs = detector.find_faces(img)

        cv2.imshow("Image", img)
        cv2.waitKey(1)


if __name__ == "__main__":
    main()





import cv2
import mediapipe as mp


class HolisticDetector:
    def __init__(self):
        self.mp_holistic = mp.solutions.holistic
        self.holistic = self.mp_holistic.Holistic(
                                            min_detection_confidence=0.5,
                                            min_tracking_confidence=0.5)

        self.mp_draw = mp.solutions.drawing_utils
        self.mp_drawing_styles = mp.solutions.drawing_styles

    def find_holistic(self, img):
        """
        When the image file is read with the OpenCV function read() ,
        the order of colors is BGR (blue, green, red). So here we are converting
        BGR image to RGB
        """
        img_rgb = cv2.cvtColor(img, cv2.COLOR_BGR2RGB)
        """
        Processes an RGB image and returns the hand landmarks and handedness of each detected hand.
        It returns a NamedTuple object with two fields: a "multi_hand_landmarks" field that
        contains the hand landmarks on each detected hand and a "multi_handedness"
        field that contains the handedness (left v.s. right hand) of the detected
        hand.
        """
        self.results = self.holistic.process(img_rgb)

        self.mp_draw.draw_landmarks(
            img,
            self.results.face_landmarks,
            self.mp_holistic.FACEMESH_TESSELATION,
            landmark_drawing_spec=None,
            connection_drawing_spec=self.mp_drawing_styles
                .get_default_face_mesh_tesselation_style())

        self.mp_draw.draw_landmarks(
            img,
            self.results.pose_landmarks,
            self.mp_holistic.POSE_CONNECTIONS,
            landmark_drawing_spec=self.mp_drawing_styles
                .get_default_pose_landmarks_style())

        self.mp_draw.draw_landmarks(
            img,
            self.results.left_hand_landmarks,
            self.mp_holistic.HAND_CONNECTIONS,
            self.mp_drawing_styles.get_default_hand_landmarks_style(),
            self.mp_drawing_styles.get_default_hand_connections_style())

        self.mp_draw.draw_landmarks(
            img,
            self.results.right_hand_landmarks,
            self.mp_holistic.HAND_CONNECTIONS,
            self.mp_drawing_styles.get_default_hand_landmarks_style(),
            self.mp_drawing_styles.get_default_hand_connections_style())

        return img


def main():
    cap = cv2.VideoCapture(0)
    detector = HolisticDetector()
    while True:
        success, img = cap.read()
        img = detector.find_holistic(img)

        cv2.imshow("Image", img)
        cv2.waitKey(1)


if __name__ == "__main__":
    main()



import cv2
import mediapipe as mp
import numpy as np


class SelfieSegmentation:
    def __init__(self):
        self.mp_selfie_segmentation = mp.solutions.selfie_segmentation
        self.selfie_segmentation = self.mp_selfie_segmentation.SelfieSegmentation(model_selection=1)

        self.mp_draw = mp.solutions.drawing_utils

    def remove_bg(self, img, img_bg=(255, 255, 255), threshold=0.1):
        """
        When the image file is read with the OpenCV function read() ,
        the order of colors is BGR (blue, green, red). So here we are converting
        BGR image to RGB
        """
        img_rgb = cv2.cvtColor(img, cv2.COLOR_BGR2RGB)
        """
        Processes an RGB image and returns the hand landmarks and handedness of each detected hand.
        It returns a NamedTuple object with two fields: a "multi_hand_landmarks" field that
        contains the hand landmarks on each detected hand and a "multi_handedness"
        field that contains the handedness (left v.s. right hand) of the detected
        hand.
        """
        self.results = self.selfie_segmentation.process(img_rgb)

        # Draw selfie segmentation on the background image.
        # To improve segmentation around boundaries, consider applying a joint
        # bilateral filter to "results.segmentation_mask" with "image".
        condition = np.stack(
            (self.results.segmentation_mask,) * 3, axis=-1) > threshold
        # The background can be customized.
        #   a) Load an image (with the same width and height of the input image) to
        #      be the background, e.g., bg_image = cv2.imread('/path/to/image/file')
        #   b) Blur the input image by applying image filtering, e.g.,
        #      bg_image = cv2.GaussianBlur(image,(55,55),0)
        if isinstance(img_bg, tuple):
            bg_image = np.zeros(img.shape, dtype=np.uint8)
            bg_image[:] = img_bg
            output_image = np.where(condition, img, bg_image)
        else:
            output_image = np.where(condition, img, img_bg)

        return output_image


def main():
    cap = cv2.VideoCapture(0)
    detector = SelfieSegmentation()
    bg_image = cv2.imread('C:/Users/praso/Downloads/1.jpg')
    while True:
        success, img = cap.read()
        img = detector.remove_bg(img, img_bg=bg_image, threshold=0.8)

        cv2.imshow("Image", img)
        cv2.waitKey(1)


if __name__ == "__main__":
    main()



from cvtech.HandTrackingModule import HandDetector
from cvtech.FaceTrackingModule import FaceDetector
from cvtech.FaceMeshModule import FaceMeshDetector
from cvtech.HumanTrackingModule import HumanDetector
from cvtech.HolisticModule import HolisticDetector
from cvtech.selfieSegmentation import SelfieSegmentation



import cv2
import mediapipe as mp
import math

class HandDetector:
    def __init__(self, detection_con=0.5, min_track_con=0.5):
        self.detectionCon = detection_con
        self.minTrackCon = min_track_con

        self.mp_hands = mp.solutions.hands
        self.hands = self.mp_hands.Hands(min_detection_confidence=self.detectionCon,
                                         min_tracking_confidence=self.minTrackCon)

        self.mp_draw = mp.solutions.drawing_utils
        self.mp_drawing_styles = mp.solutions.drawing_styles

        self.tip_ids = [4, 8, 12, 16, 20]
        self.fingerName = ["Thumb", "Index Finger", "Middle Finger", "Ring Finger", "Little Finger"]

    def find_hands(self, img, draw=True, flip_type=True):
        """
        When the image file is read with the OpenCV function read() ,
        the order of colors is BGR (blue, green, red). So here we are converting
        BGR image to RGB
        """
        img_rgb = cv2.cvtColor(img, cv2.COLOR_BGR2RGB)
        """
        Processes an RGB image and returns the hand landmarks and handedness of each detected hand.
        It returns a NamedTuple object with two fields: a "multi_hand_landmarks" field that
        contains the hand landmarks on each detected hand and a "multi_handedness"
        field that contains the handedness (left v.s. right hand) of the detected
        hand.
        """
        self.results = self.hands.process(img_rgb)
        """
        hold the properties of all the hands
        """
        all_hands = []
        height, width, color = img.shape
        BBOX_OFFSET = 15

        if self.results.multi_hand_landmarks:
            """
            hand_type will contain the details of hands. {Right Hand , Left Hand}
            hand_lms will contain the landmarks of hands. one hand contains 21 landmarks.
            iterates on each hands at once
            """
            for hand_type, hand_lms in zip(self.results.multi_handedness, self.results.multi_hand_landmarks):
                """
                it will contain the different properties of hands
                lm_list--> coordinates of landmarks. (x,y)
                bbox--> coordinates of boundary box.
                """
                my_hand={}
                """
                it will contain the coordinates of landmarks
                lm_list--> coordinates of landmarks. (x,y)
                """
                mylm_list = []
                """ 
                it contains the x coordinates of landmarks
                """
                x_list = []
                """
                it contains the y coordinates of landmarks.
                """
                y_list = []
                """
                Extracts the landmarks from each hands and fit those landmarks with image shape (e.g. 640x480)
                and store it in mylm_list
                """
                for id, lm in enumerate(hand_lms.landmark):
                    px, py = int(lm.x * width), int(lm.y * height)
                    mylm_list.append([px, py])
                    x_list.append(px)
                    y_list.append(py)

                my_hand["lm_list"] = mylm_list

                """create a boundary box around the  each hand."""
                xmin, xmax = min(x_list), max(x_list)
                ymin, ymax = min(y_list), max(y_list)
                bbox = xmin, ymin, xmax, ymax

                my_hand["bbox"] = bbox

                if flip_type:
                    if hand_type.classification[0].label == "Right":
                        my_hand["type"] = "Left"
                    else:
                        my_hand["type"] = "Right"
                else:
                    my_hand["type"] = hand_type.classification[0].label

                all_hands.append(my_hand)

                """Draw the hands"""
                if draw:
                    self.mp_draw.draw_landmarks(img, hand_lms,
                                                self.mp_hands.HAND_CONNECTIONS,
                                                self.mp_drawing_styles.get_default_hand_landmarks_style())

                    cv2.rectangle(img, (bbox[0]-BBOX_OFFSET, bbox[3]+BBOX_OFFSET),
                                  (bbox[2]+BBOX_OFFSET, bbox[1]-BBOX_OFFSET), (0, 255, 0), 2)

                    cv2.putText(img, my_hand["type"], (bbox[0], bbox[1]-30),
                                cv2.FONT_HERSHEY_SIMPLEX, 1, (255, 0, 0), 2)

        if draw:
            return all_hands, img
        else:
            return all_hands

    def fingers_up(self, my_hand):
        my_hand_type = my_hand["type"]
        mylm_list = my_hand["lm_list"]

        if self.results.multi_hand_landmarks:
            fingers = []

            ## Thumb
            if my_hand_type == "Right":
                if mylm_list[self.tip_ids[0]][0] > mylm_list[self.tip_ids[0] - 1][0]:
                    fingers.append(1)
                else:
                    fingers.append(0)
            else:
                if mylm_list[self.tip_ids[0]][0] < mylm_list[self.tip_ids[0] - 1][0]:
                    fingers.append(1)
                else:
                    fingers.append(0)

            ## Fingers
            for id in range(1, 5):
                if mylm_list[self.tip_ids[id]][1] < mylm_list[self.tip_ids[id] - 2][1]:
                    fingers.append(1)
                else:
                    fingers.append(0)

        return fingers

    def find_distance(self, p1, p2, img=None):
        x1, y1 = p1
        x2, y2 = p2
        cx, cy = (x1 + x2) // 2, (y1 + y2) // 2
        length = math.hypot(x2 - x1, y2 - y1)
        info = (x1, y1, x2, y2, cx, cy)
        if img is not None:
            cv2.circle(img, (x1, y1), 5, (0, 0, 255), cv2.FILLED)
            cv2.circle(img, (x2, y2), 5, (0, 0, 255), cv2.FILLED)
            cv2.line(img, (x1, y1), (x2, y2), (0, 0, 255), 3)
            cv2.circle(img, (cx, cy), 10, (0, 0, 255), cv2.FILLED)
            return length, info, img
        else:
            return length, info


def main():
    cap = cv2.VideoCapture(0)
    detector = HandDetector()
    while True:
        success, img = cap.read()
        hands, img = detector.find_hands(img)

        if hands:
            hand1 = hands[0]
            lm_list1 = hand1["lm_list"]
            fingers1 = detector.fingers_up(hand1)

            if len(hands) == 2:
                hand2 = hands[1]
                lm_list2 = hand2["lm_list"]
                fingers2 = detector.fingers_up(hand2)
                length, info, img = detector.find_distance(lm_list1[8], lm_list2[8], img)

        cv2.imshow("Image", img)
        cv2.waitKey(1)


if __name__ == "__main__":
    main()



import cv2
import mediapipe as mp


class FaceMeshDetector:
    def __init__(self):
        self.mp_draw = mp.solutions.drawing_utils
        self.mp_drawing_styles = mp.solutions.drawing_styles
        self.mp_face_mesh = mp.solutions.face_mesh

        self.drawing_spec = self.mp_draw.DrawingSpec(thickness=0.5, circle_radius=1)
        self.faceMesh = self.mp_face_mesh.FaceMesh(max_num_faces=1,
                                                   refine_landmarks=True,
                                                   min_detection_confidence=0.5,
                                                   min_tracking_confidence=0.5)

    def find_faceMesh(self, img, draw=True):
        """
        When the image file is read with the OpenCV function read() ,
        the order of colors is BGR (blue, green, red). So here we are converting
        BGR image to RGB
        """
        img_rgb = cv2.cvtColor(img, cv2.COLOR_BGR2RGB)
        """
        Processes an RGB image and returns a list of the detected face location data.
        It returns a NamedTuple object with a "detections" field that contains a list of the
        detected face location data.
        """
        self.results = self.faceMesh.process(img_rgb)
        faces = []

        if self.results.multi_face_landmarks:
            for face_landmarks in self.results.multi_face_landmarks:
                if draw:
                    self.mp_draw.draw_landmarks(
                        image=img,
                        landmark_list=face_landmarks,
                        connections=self.mp_face_mesh.FACEMESH_TESSELATION,
                        landmark_drawing_spec=None,
                        connection_drawing_spec=self.mp_drawing_styles.get_default_face_mesh_tesselation_style())

                    self.mp_draw.draw_landmarks(
                        image=img,
                        landmark_list=face_landmarks,
                        connections=self.mp_face_mesh.FACEMESH_CONTOURS,
                        landmark_drawing_spec=None,
                        connection_drawing_spec=self.mp_drawing_styles.get_default_face_mesh_contours_style())

                    self.mp_draw.draw_landmarks(
                        image=img,
                        landmark_list=face_landmarks,
                        connections=self.mp_face_mesh.FACEMESH_IRISES,
                        landmark_drawing_spec=None,
                        connection_drawing_spec=self.mp_drawing_styles.get_default_face_mesh_iris_connections_style())

                face = []
                for id, lm in enumerate(face_landmarks.landmark):
                    ih, iw, ic = img.shape
                    x, y = int(lm.x * iw), int(lm.y * ih)
                    face.append([x, y])
                faces.append(face)

            return img, faces


def main():
    cap = cv2.VideoCapture(0)
    detector = FaceMeshDetector()
    while True:
        success, img = cap.read()
        img, faces = detector.find_faceMesh(img, draw=False)

        if faces:
            print(faces[0])

        cv2.imshow("Image", img)
        cv2.waitKey(1)


if __name__ == "__main__":
    main()



import cv2
import mediapipe as mp


class HumanDetector:
    def __init__(self):
        self.mp_pose = mp.solutions.pose
        self.pose = self.mp_pose.Pose(
                                    min_detection_confidence=0.5,
                                    min_tracking_confidence=0.5)

        self.mp_draw = mp.solutions.drawing_utils
        self.mp_drawing_styles = mp.solutions.drawing_styles

    def find_humans(self, img):
        """
        When the image file is read with the OpenCV function read() ,
        the order of colors is BGR (blue, green, red). So here we are converting
        BGR image to RGB
        """
        img_rgb = cv2.cvtColor(img, cv2.COLOR_BGR2RGB)
        """
        Processes an RGB image and returns the hand landmarks and handedness of each detected hand.
        It returns a NamedTuple object with two fields: a "multi_hand_landmarks" field that
        contains the hand landmarks on each detected hand and a "multi_handedness"
        field that contains the handedness (left v.s. right hand) of the detected
        hand.
        """
        self.results = self.pose.process(img_rgb)

        if self.results.pose_landmarks:
            self.mp_draw.draw_landmarks(
                img,
                self.results.pose_landmarks,
                self.mp_pose.POSE_CONNECTIONS,
                landmark_drawing_spec=self.mp_drawing_styles.get_default_pose_landmarks_style())

        return img


def main():
    cap = cv2.VideoCapture(0)
    detector = HumanDetector()
    while True:
        success, img = cap.read()
        img = detector.find_humans(img)

        cv2.imshow("Image", img)
        cv2.waitKey(1)


if __name__ == "__main__":
    main()
